;=======================================================================
; Sideways Growing pipe
; By: Erik
;
; Description: This is a version of the shrinking/growing pipe that
; grows to the left/right instead. Based on imamelia's disassembly.
;=======================================================================
; Uses first extra bit: YES
; If the extra bit is set, the pipe will start out going to the right.
;=======================================================================


x_speed:                        ;   speeds at which the pipe grows
        db $00,$F0,$00,$10

frame_count:                    ;   how much time before it starts moving again
        db $20,$40,$20,$40

!pipe_side_1    = $013D         ;\  map16 tiles the pipe draws.
!pipe_side_2    = $013E         ;/

!pipe_tile_1    = $A4           ;\  the pipe's tilemap.
!pipe_tile_2    = $A6           ;/

print "INIT ",pc
        LDA !7FAB10,x   ;
        AND #$04        ; if the extra bit is set...
        BEQ +           ;
        INC !C2,x       ; make the sprite start out in state 02
        INC !C2,x       ; so that it goes down instead
+       RTL

print "MAIN ",pc
        PHB
        PHK
        PLB
        JSR sideways_pipe
        PLB
        RTL

tiles_to_generate:              ;   don't touch.
        db $00,$01,$00,$02

sideways_pipe:
        JSR graphics            ;   draw the sprite's graphics
        LDA #$00                ;\  process offscreen
        %SubOffScreen()         ;/
        LDA $9D                 ;\  if sprites or locked...
        ORA !15A0,x             ; | or the sprite is offscreen horizontally...
        BNE no_update_pos       ;/  don't update its position
        %SubVertPos()           ;
        LDA $0F                 ;
        CLC                     ;
        ADC #$50                ; if the sprite is more than 5 tiles away from the player...
        CMP #$A0                ;
        BCS no_update_pos       ; don't update the sprite's position
        LDA !C2,x               ;
        AND #$03                ;
        TAY                     ;
        INC !1570,x             ;
        LDA !1570,x             ; after a certain number of frames...
        CMP frame_count,y       ;
        BNE .no_state_change    ;
        STZ !1570,x             ;
        INC !C2,x               ; change the sprite state
        BRA no_update_pos       ;

.no_state_change
        LDA x_speed,y           ;\
        STA !B6,x               ; | if the sprite X speed is nonzero...
        BEQ .not_generate       ; |
        LDA !E4,x               ; |
        AND #$0F                ; | and the sprite is centered over a tile...
        BNE .not_generate       ; |
        JSR generate_tiles      ;/  generate pipe tiles

.not_generate
        JSL $018022|!BankB      ;   update sprite Y position
no_update_pos:
        PEI ($94)
        JSL $01B44F|!BankB      ;   make the sprite solid
        PLA
        STA $94
        PLA
        STA $95
        JSL $01A7DC|!BankB
        BCC return_interact
        %SubVertPos()
        LDA $0F
        DEY
        BPL return_interact
        %SubHorzPos()
        LDA $0E
        CMP #$08
        BCC +
        CMP #$F8
        BCS return_interact
        ; CMP #$F3
        ; BCC return_interact
        ; CMP #$F6
        ; BCS return_interact
+       LDA !14F8,x
        PHA
        LDA !E4,x
        PHA
        LDA !14E0,x
        PHA
        JSL $018022|!BankB      ;   "update" sprite Y position
        PLA
        STA !14E0,x
        PLA
        STA !E4,x
        PLA
        STA !14F8,x
        LDY #$00
        LDA $1491|!Base2
        BPL +
        DEY
+       CLC
        ADC $94
        STA $94
        TYA
        ADC $95
        STA $95
        STZ $7B
return_interact:
        RTS

;==================
  generate_tiles:
;==================

        LDA !7FAB10,x
        AND #$04
        BEQ .not_right
        TYA
        INC #2
        AND #$03
        TAY
.not_right
        LDA tiles_to_generate,y
        BEQ .no_generate
        LSR
        BCS .generate_pipe
.generate_blank
        LDA #$02
        STA $9C
        LDA !E4,x
        STA $9A
        LDA !14E0,x
        STA $9B
        LDA !D8,x
        STA $98
        LDA !14D4,x
        STA $99
        JSL $00BEB0|!BankB
        LDA #$02
        STA $9C
        LDA !E4,x
        STA $9A
        LDA !14E0,x
        STA $9B
        LDA !D8,x
        CLC
        ADC #$10
        STA $98
        LDA !14D4,x
        ADC #$00
        STA $99
        JSL $00BEB0|!BankB
.no_generate
        RTS

.generate_pipe
        LDA !E4,x
        STA $9A
        LDA !14E0,x
        STA $9B
        LDA !D8,x
        STA $98
        LDA !14D4,x
        STA $99
        REP #$30
        LDA.w #!pipe_side_1
        %ChangeMap16()
        SEP #$30
        LDA !E4,x
        STA $9A
        LDA !14E0,x
        STA $9B
        LDA !D8,x
        CLC
        ADC #$10
        STA $98
        LDA !14D4,x
        ADC #$00
        STA $99
        REP #$30
        LDA.w #!pipe_side_2
        %ChangeMap16()
        SEP #$30
        RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; graphics routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

graphics:
        %GetDrawInfo()
        LDA $00
        STA $0300|!Base2,y
        STA $0304|!Base2,y
        LDA $01
        DEC
        STA $0301|!Base2,y
        CLC
        ADC #$10
        STA $0305|!Base2,y
        LDA #!pipe_tile_1
        STA $0302|!Base2,y
        LDA #!pipe_tile_2
        STA $0306|!Base2,y
        LDA !15F6,x
        ORA $64
        STA $0303|!Base2,y
        STA $0307|!Base2,y
        LDY #$02
        LDA #$01
        JSL $01B7B3|!BankB
        RTS

